from string import ascii_uppercase, ascii_lowercase, digits, punctuation

EMPTY_STRING: str = ""
DEFAULT_SEPARATOR = "-"
DEFAULT_SEPARATE_LENGTH: int = 4
PASSWORD_LENGTH_LIMIT: int = 72
DEFAULT_MIN_PASS_LEN: int = 8
DEFAULT_MAX_PASS_LEN: int = 16
NO_SHORT: str = "n"
YES_SHORT: str = "y"
POSSIBLE_UPPERCASE_CHARS: str = ascii_uppercase
POSSIBLE_LOWERCASE_CHARS: str = ascii_lowercase
POSSIBLE_NUMBERS: str = digits
POSSIBLE_SPECIAL_CHARS: str = punctuation
